// Description: 
// By Sarah Dean
// Email: sdean12@softhome.net
// WWW:   http://www.FreeOTFE.org/
//
// -----------------------------------------------------------------------------
//


#include "FreeOTFEHashImpl.h"
#include "FreeOTFEHashRIPEMD.h"

#include "FreeOTFEDebug.h"
#include "FreeOTFElib.h"


// Include TomLibCrypt libraries...
#include <mycrypt_custom.h>


// =========================================================================
// Hash driver init function
// devExt - The device extension to be initialized
NTSTATUS
ImpHashDriverExtDetailsInit(
    IN OUT DEVICE_EXTENSION* devExt
)
{
    NTSTATUS status = STATUS_SUCCESS;
    int idx = -1;

    DEBUGOUTHASHIMPL(DEBUGLEV_ENTER, ("ImpHashDriverExtDetailsInit\n"));


    // -- POPULATE DRIVER IDENTIFICATION --
    RtlZeroMemory(devExt->DriverTitle, sizeof(devExt->DriverTitle));
    RtlCopyMemory(
                  devExt->DriverTitle,
                  DRIVER_TITLE,
                  strlen(DRIVER_TITLE)
                 );

    devExt->DriverGUID = DRIVER_GUID;
    devExt->DriverVersionID = DRIVER_HASH_VERSION;


    // -- POPULATE HASHES SUPPORTED --
    devExt->HashDetails = ExAllocatePool(
                                         NonPagedPool,
                                         (sizeof(HASH) * HASHES_SUPPORTED)
                                        );    

    devExt->HashCount = HASHES_SUPPORTED;



    // -- RIPEMD-128 --
    idx++;
    RtlZeroMemory(devExt->HashDetails[idx].Title, MAX_HASH_TITLE);
    RtlCopyMemory(
                  devExt->HashDetails[idx].Title,
                  DRIVER_HASH_TITLE_RMD128,
                  strlen(DRIVER_HASH_TITLE_RMD128)
                 );

    devExt->HashDetails[idx].Length = (rmd128_desc.hashsize * 8);
    devExt->HashDetails[idx].BlockSize = (rmd128_desc.blocksize * 8);
    devExt->HashDetails[idx].VersionID = DRIVER_HASH_IMPL_VERSION;
    devExt->HashDetails[idx].HashGUID = HASH_GUID_RMD128;


    // -- RIPEMD-160 --
    idx++;
    RtlZeroMemory(devExt->HashDetails[idx].Title, MAX_HASH_TITLE);
    RtlCopyMemory(
                  devExt->HashDetails[idx].Title,
                  DRIVER_HASH_TITLE_RMD160,
                  strlen(DRIVER_HASH_TITLE_RMD160)
                 );

    devExt->HashDetails[idx].Length = (rmd160_desc.hashsize * 8);
    devExt->HashDetails[idx].BlockSize = (rmd160_desc.blocksize * 8);
    devExt->HashDetails[idx].VersionID = DRIVER_HASH_IMPL_VERSION;
    devExt->HashDetails[idx].HashGUID = HASH_GUID_RMD160;


    // -- RIPEMD-160, twice with A; Linux --
    idx++;
    RtlZeroMemory(devExt->HashDetails[idx].Title, MAX_HASH_TITLE);
    RtlCopyMemory(
                  devExt->HashDetails[idx].Title,
                  DRIVER_HASH_TITLE_RMD160_2xA_LINUX,
                  strlen(DRIVER_HASH_TITLE_RMD160_2xA_LINUX)
                 );

    devExt->HashDetails[idx].Length = (rmd160_desc.hashsize * 8) * 2;
    devExt->HashDetails[idx].BlockSize = (rmd160_desc.blocksize * 8);
    devExt->HashDetails[idx].VersionID = DRIVER_HASH_IMPL_VERSION;
    devExt->HashDetails[idx].HashGUID = HASH_GUID_RMD160_2xA_LINUX;


    DEBUGOUTHASHIMPL(DEBUGLEV_EXIT, ("ImpHashDriverExtDetailsInit\n"));

    return status;
}


// =========================================================================
// Hash driver cleardown function
// devExt - The device extension to be cleared down
NTSTATUS
ImpHashDriverExtDetailsCleardown(    
    IN OUT DEVICE_EXTENSION* devExt
)
{
    NTSTATUS status = STATUS_SUCCESS;

    DEBUGOUTHASHIMPL(DEBUGLEV_ENTER, ("ImpHashDriverExtDetailsCleardown\n"));

    if (devExt->HashDetails != NULL)
        {
        ExFreePool(devExt->HashDetails);
        }

    devExt->HashDetails = NULL;
    devExt->HashCount = 0;

    DEBUGOUTHASHIMPL(DEBUGLEV_EXIT, ("ImpHashDriverExtDetailsCleardown\n"));

    return status;
}


// =========================================================================
// Hash function
NTSTATUS
ImpHashHashData(
    IN      GUID* HashGUID,
    IN      unsigned int DataLength,  // In bits
    IN      unsigned char* Data,
    IN OUT  unsigned int* HashLength,  // In bits
    OUT     unsigned char* Hash
)
{
    NTSTATUS status = STATUS_SUCCESS;
    hash_state md;
    unsigned int ile;
    unsigned char pwdCopy[130];

    DEBUGOUTHASHIMPL(DEBUGLEV_ENTER, ("ImpHashHashData\n"));
    if (IsEqualGUID(&HASH_GUID_RMD160, HashGUID))
        {
        if (*HashLength < (rmd160_desc.hashsize * 8))
            {
            DEBUGOUTHASHIMPL(DEBUGLEV_ERROR, ("output hash length buffer too small (got: %d; need: %d)\n",
                *HashLength,
                (rmd160_desc.hashsize * 8)
                ));
            status = STATUS_BUFFER_TOO_SMALL;
            }
        else
            {
            rmd160_desc.init(&md);
            rmd160_desc.process(&md, Data, (DataLength / 8));
            rmd160_desc.done(&md, Hash);
        
            *HashLength = (rmd160_desc.hashsize * 8);
            }
            
        }
    else if (IsEqualGUID(&HASH_GUID_RMD128, HashGUID))
        {
        if (*HashLength < (rmd128_desc.hashsize * 8))
            {
            DEBUGOUTHASHIMPL(DEBUGLEV_ERROR, ("output hash length buffer too small (got: %d; need: %d)\n",
                *HashLength,
                (rmd128_desc.hashsize * 8)
                ));
            status = STATUS_BUFFER_TOO_SMALL;
            }
        else
            {
            rmd128_desc.init(&md);
            rmd128_desc.process(&md, Data, (DataLength / 8));
            rmd128_desc.done(&md, Hash);
        
            *HashLength = (rmd128_desc.hashsize * 8);
            }
            
        }
    else if (IsEqualGUID(&HASH_GUID_RMD160_2xA_LINUX, HashGUID))
        {
        if (*HashLength < ((rmd160_desc.hashsize * 8) * 2))
            {
            DEBUGOUTHASHIMPL(DEBUGLEV_ERROR, ("output hash length buffer too small (got: %d; need: %d)\n",
                *HashLength,
                ((rmd160_desc.hashsize * 8) * 2)
                ));
            status = STATUS_BUFFER_TOO_SMALL;
            }
        else
            {
            rmd160_desc.init(&md);
            rmd160_desc.process(&md, Data, (DataLength / 8));
            rmd160_desc.done(&md, Hash);

            pwdCopy[0] = 'A';

            ile = (DataLength / 8);
            if (ile > sizeof(pwdCopy) - 1) 
                {
                ile = sizeof(pwdCopy) - 1;
                }

            RtlCopyMemory(
                          pwdCopy + 1,
                          Data,
                          ile
                         );

            rmd160_desc.init(&md);
            rmd160_desc.process(&md, pwdCopy, (ile + 1));
            rmd160_desc.done(&md, &Hash[20]);

            *HashLength = ((rmd160_desc.hashsize * 8) * 2);
            }
            
        }
    else
        {
        DEBUGOUTHASHIMPL(DEBUGLEV_ERROR, ("driver doesn't recognise GUID\n"));
        status = STATUS_INVALID_PARAMETER;
        }
        
    DEBUGOUTHASHIMPL(DEBUGLEV_EXIT, ("ImpHashHashData\n"));
      
    return status;
}


// =========================================================================
// =========================================================================

